%====================================================================
%           Balanced Perfect Ternary Sequence Set
%
%Inputs
%------
%   1. N    : sequence length (31 or 127)
%   2. CMAX : maximum desired crosscorrelation level
%           : This will determine the size of the sequence set 
%           : CMAX = 4 or 6 for sequences of length 31
%           : CMAX = 8,12,16 or 20 for sequences of length 127
%Outputs
%-------
%   1. Coherent Auto/Cross-correlation matrix
%   2. NonCoherent Auto/Cross-correlation matrix
%   3. Coherent Periodic Autocorrelation Function
%   4. Coherent Periodic Crosscorrelation Function
%   5. Non-Coherent Periodic Autocorrelation Function
%   6. Non-Coherent Periodic Crosscorrelation Function
%   7. Display the sequence set
%   8. Shows one period of the preamble
%====================================================================
%Authors    : Ismail Lakkis
%Companies  : Wideband Access & I2R
%Date       : July 11, 2005
%Verison    : 1
%====================================================================

clear all;
format long;

%parameters
%==========
N   = 127;      %sequence length
CMAX= 8;        %maximum desired cross-correlation level

if (N == 31), load x31; x = x31; clear x31;
else load x127; x = x127; clear x127;end;

if (N == 31),
    switch CMAX,
        case 4,     J = [1 7 10];
        otherwise,  J = [1 7 10 4 5 12]; %CMAX = 6
    end;
else
    switch CMAX,
        case 8,     J = [1 22 30];
        case 12,    J = [1 22 30 7];
        case 16,    J = [1 22 7 31 61];
        otherwise,  J = [1 2 7 8 15 16 17 18 27 28 31 32 63 64];   %CMAX = 20
    end;
end;
x = x(J,:);L = length(J);

%compute cross-correlation matrix
%================================
CC = zeros(L,L);CN = zeros(L,L);
for k = 1:L,
    a = x(k,:);Hc = hankel(a(1:N),[a(N) a(1:N-1)]);
    b = a.^2;Hn = hankel(b(1:N),[b(N) b(1:N-1)]);
    for m = 1:L,
        CC(k,m) = max(abs(x(m,:)*Hc));c = 2*(x(m,:).^2)-1;CN(k,m) = max(abs(c*Hn));
    end;
end;
fprintf('Coherent cross-correlation matrix     =\n');
for k = 1:L,
    for m = 1:L, fprintf(' %d',CC(k,m));end;fprintf('\n');
end;
fprintf('Non-coherent cross-correlation matrix =\n');  
for k = 1:L,
    for m = 1:L, fprintf(' %d',CN(k,m));end;fprintf('\n');
end;

%plot results
%============
H = hankel(x(1,1:N),[x(1,N) x(1,1:N-1)]);
figure;stem(x(1,:)*H);grid;title('Coherent Receiver: Periodic Autocorrelation Function');
figure;stem(x(2,:)*H);grid;title('Coherent Receiver: Periodic Cross-correlation Function');

%noncoherent
%===========
z1 = x(1,:).^2;z2 = x(2,:).^2;H = hankel(z1(1:N),[z1(N) z1(1:N-1)]);
figure;stem((2*z1-1)*H);grid;title('Non-Coherent Receiver: Periodic Autocorrelation Function');
figure;stem((2*z2-1)*H);grid;title('Non-Coherent Receiver: Periodic Cross-correlation Function');

%display sequences
%=================
for k = 1:L,
    fprintf('s');
    fprintf('%d',k);
    fprintf(' = ');
    for m = 1:N,
        switch x(k,m),case 1, fprintf('+');case 0, fprintf('0');otherwise fprintf('-');end;
    end;
    fprintf('\n');
end;

%modulated data
%==============
y = kron(x(1,:),[1 zeros(1,63)]);
p = sinc([-2:1/8:2]);
y = conv(p,y);Ly = length(y);ii = find(y == max(y));t = [1:Ly] - ii(1);t = t/64;
figure;
plot(t,y);grid;title('impulse radio: one period of s1(t)');xlabel('t/PRI');
    